
/**

GNU-DiaboloSimulator : ce programme permet de simuler des siteswaps au
diabolo.

Copyright (C) 2012 Frédéric MICOUT - fred@sujets-libres.fr
Tous droits réservés.

Ce programme est un logiciel libre ; vous pouvez le redistribuer ou le
modifier suivant les termes de la “GNU General Public License” telle que
publiée par la Free Software Foundation : soit la version 3 de cette
licence, soit (à votre gré) toute version ultérieure.

Ce programme est distribué dans l’espoir qu’il vous sera utile, mais SANS
AUCUNE GARANTIE : sans même la garantie implicite de COMMERCIALISABILITÉ
ni d’ADÉQUATION À UN OBJECTIF PARTICULIER. Consultez la Licence Générale
Publique GNU pour plus de détails.

Vous devriez avoir reçu une copie de la Licence Générale Publique GNU avec
ce programme ; si ce n’est pas le cas, consultez :
<http://www.gnu.org/licenses/>.

**/

#include "siteswap.h"
#include "animation.h"
#include "mywidget.h"
#include "qmovablegraphicsview.h"

mywidget::mywidget(QWidget *parent) :
    QWidget(parent)
{
    int i;

    /** Création des objets et premiers agencements **/

    // Saisie d'informations
    QGroupBox *saisieSiteswap = new QGroupBox("Siteswap", this);
    saisieSiteswap->setFont(QFont("Times", 12, QFont::DemiBold));
    QLineEdit *champTexte = new QLineEdit;
    champTexte->setText(" "); // texte fixé une première fois dans le champs texte à sa création (à cause d'un bug)
    QHBoxLayout *hboxSaisie = new QHBoxLayout();
    hboxSaisie->addWidget(champTexte);
    saisieSiteswap->setLayout(hboxSaisie);



    // Options jonglerie de l'animation
    QGroupBox *lateralisation = new QGroupBox("Latéralisation", this);
    lateralisation->setFont(QFont("Times", 12, QFont::DemiBold));
    QRadioButton *droitier = new QRadioButton("droitier");
    QRadioButton *gaucher = new QRadioButton("gaucher");
    droitier->setChecked(true);
    QVBoxLayout *vboxLateralisation = new QVBoxLayout();
    vboxLateralisation->addWidget(droitier);
    vboxLateralisation->addWidget(gaucher);
    lateralisation->setLayout(vboxLateralisation);

    QGroupBox *hauteur = new QGroupBox("Mode", this);
    hauteur->setFont(QFont("Times", 12, QFont::DemiBold));
    QRadioButton *high = new QRadioButton("aérien");
    QRadioButton *low = new QRadioButton("corde");
    high->setChecked(true);
    QVBoxLayout *vboxHauteur = new QVBoxLayout();
    vboxHauteur->addWidget(high);
    vboxHauteur->addWidget(low);
    hauteur->setLayout(vboxHauteur);

    QHBoxLayout *hboxBoutonsRadio = new QHBoxLayout();
    hboxBoutonsRadio->addWidget(lateralisation);
    hboxBoutonsRadio->addWidget(hauteur);



    // Gestion du temps
    QGroupBox *temps = new QGroupBox("Gestion du temps", this);
    temps->setFont(QFont("Times", 12, QFont::DemiBold));
    QVBoxLayout *vboxTemps = new QVBoxLayout();

    QHBoxLayout *hboxTempsDeBase = new QHBoxLayout();
    QLabel *labTempsBase = new QLabel;
    labTempsBase->setFont(QFont("Times", 9, QFont::DemiBold));
    labTempsBase->setFixedWidth(130);
    labTempsBase->setText("temps de base (ms)");
    QSlider *sliderTempsBase = new QSlider(Qt::Horizontal,this);
    sliderTempsBase->setMinimum(500);
    sliderTempsBase->setMaximum(1300);
    sliderTempsBase->setValue(900);
    QLabel *valTempsBase = new QLabel;
    valTempsBase->setFont(QFont("Times", 9, QFont::DemiBold));
    valTempsBase->setFixedWidth(25);
    valTempsBase->setText("900"); // Recopie de la valeur de initiale du slider
    hboxTempsDeBase->addWidget(labTempsBase);
    hboxTempsDeBase->addWidget(sliderTempsBase);
    hboxTempsDeBase->addWidget(valTempsBase);

    QHBoxLayout *hboxTempsVide = new QHBoxLayout();
    QLabel *labTempsVide = new QLabel;
    labTempsVide->setFont(QFont("Times", 9, QFont::DemiBold));
    labTempsVide->setFixedWidth(130);
    labTempsVide->setText("coef corde à vide (%)");
    QSlider *sliderTempsVide = new QSlider(Qt::Horizontal,this);
    sliderTempsVide->setMinimum(0);
    sliderTempsVide->setMaximum(80);
    sliderTempsVide->setValue(40);
    QLabel *valTempsVide = new QLabel;
    valTempsVide->setFont(QFont("Times", 9, QFont::DemiBold));
    valTempsVide->setFixedWidth(25);
    valTempsVide->setText("40"); // Recopie de la valeur de initiale du slider
    hboxTempsVide->addWidget(labTempsVide);
    hboxTempsVide->addWidget(sliderTempsVide);
    hboxTempsVide->addWidget(valTempsVide);

    vboxTemps->addLayout(hboxTempsDeBase);
    vboxTemps->addLayout(hboxTempsVide);
    temps->setLayout(vboxTemps);



    // Options des diabolos (affichage nombre et modification couleur) dans l'animation
    QGroupBox *diabolos = new QGroupBox("Couleur des diabolos", this);
    diabolos->setFont(QFont("Times", 12, QFont::DemiBold));
    QHBoxLayout *hboxDiabolos = new QHBoxLayout();

    QLabel *siteSNombreObj = new QLabel; // Pour affichage du nombre d'objets du siteswap -- Plus utilisé en tant que tel
    siteSNombreObj->setFont(QFont("Times", 12, QFont::DemiBold));
    siteSNombreObj->setText("3");

    affichageDiabolos.push_back(new QPushButton());
    affichageDiabolos.push_back(new QPushButton());
    affichageDiabolos.push_back(new QPushButton());
    affichageDiabolos.push_back(new QPushButton());
    affichageDiabolos.push_back(new QPushButton());
    affichageDiabolos.push_back(new QPushButton());
    affichageDiabolos.push_back(new QPushButton());
    affichageDiabolos.push_back(new QPushButton());
    affichageDiabolos.push_back(new QPushButton());

    QString style;
    for(i=0;i<9;i++)
    {
        style = "background-color: rgb(100,100,100)";
        affichageDiabolos.at(i)->setStyleSheet(style);
        couleurDiabolos.push_back(QColor((255*(2-i/3))%256,(80*i)%256,(255*(i/3))%256));
    }

    hboxDiabolos->addWidget(affichageDiabolos.at(0));
    hboxDiabolos->addWidget(affichageDiabolos.at(1));
    hboxDiabolos->addWidget(affichageDiabolos.at(2));
    hboxDiabolos->addWidget(affichageDiabolos.at(3));
    hboxDiabolos->addWidget(affichageDiabolos.at(4));
    hboxDiabolos->addWidget(affichageDiabolos.at(5));
    hboxDiabolos->addWidget(affichageDiabolos.at(6));
    hboxDiabolos->addWidget(affichageDiabolos.at(7));
    hboxDiabolos->addWidget(affichageDiabolos.at(8));

    diabolos->setLayout(hboxDiabolos);



    // Affichage d'informations calculées à partir de la saisie
    SiteSwap *siteswap = new SiteSwap();



    // Affichage de l'animation
    Animation *animation = new Animation();



    // Boutons généraux
    QPushButton *valider = new QPushButton(tr("Valider"));
    valider->setFont(QFont("Times", 12, QFont::DemiBold));
    QPushButton *quit = new QPushButton(tr("Quitter"));
    quit->setFont(QFont("Times", 12, QFont::DemiBold));
    QHBoxLayout *boutons = new QHBoxLayout();
    boutons->addWidget(valider);
    boutons->addWidget(quit);


    /** Connection signaux -> slots **/

    // Action des boutons
    connect(quit, SIGNAL(clicked()), qApp, SLOT(quit()));

    // Gestion des sliders (affichage de la valeur dans la fenêtre principale et prise en compte de ces valeurs par l'animation)
    connect(sliderTempsBase,SIGNAL(valueChanged(int)),valTempsBase,SLOT(setNum(int)));
    connect(sliderTempsVide,SIGNAL(valueChanged(int)),valTempsVide,SLOT(setNum(int)));
    connect(sliderTempsBase,SIGNAL(valueChanged(int)),animation,SLOT(setTempsBase(int)));
    connect(sliderTempsVide,SIGNAL(valueChanged(int)),animation,SLOT(setPourcentageTempsAVide(int)));

    // Envoi d'information à l'objet siteswap à chaque modification du champ texte
    connect(champTexte, SIGNAL(textChanged(QString)), siteswap, SLOT(textChanged(QString)));
    connect(champTexte, SIGNAL(textChanged(QString)), animation, SLOT(textChanged(QString)));
    champTexte->setText(""); // texte fixé une seconde fois dans le champs texte après une première évaluation par le signal "textChanged" (à cause d'un bug)

    // Mise à jour des informations (période et nombre d'objets) à chaque modification
    connect(siteswap, SIGNAL(getLabNbObjets(QString)), siteSNombreObj, SLOT(setText(QString)));

    // Validation de la séquence afin d'afficher l'animation du siteswap
    connect(valider, SIGNAL(clicked()), animation, SLOT(setAnimation()));
    connect(champTexte, SIGNAL(returnPressed()), animation, SLOT(setAnimation()));
    connect(high, SIGNAL(clicked()), animation, SLOT(setHigh()));
    connect(low, SIGNAL(clicked()), animation, SLOT(setLow()));
    connect(droitier, SIGNAL(clicked()), animation, SLOT(setDroitier()));
    connect(gaucher, SIGNAL(clicked()), animation, SLOT(setGaucher()));

    // Envoi d'information à l'objet animation à chaque modification du champ texte
    connect(siteswap, SIGNAL(getSequence(QList<int>)), animation, SLOT(siteswapChanged(QList<int>)));
    connect(siteswap, SIGNAL(getPremierePosition(QList<int>)), animation, SLOT(tempsDeDepartParObjets(QList<int>)));
    connect(siteswap, SIGNAL(getSequenceParObjet(QList<QList<int> >)), animation, SLOT(sequenceParObjetChange(QList<QList<int> >)));
    connect(siteswap, SIGNAL(getValiditeSequence(bool)), animation, SLOT(siteswapValide(bool)));
    connect(siteswap, SIGNAL(getNbObjetsSequence(int)), animation, SLOT(siteswapNbObjets(int)));

    // Affichage en couleur des diabolos utilisés dans la fenêtre principale
    connect(siteswap, SIGNAL(getNbObjetsSequence(int)), this, SLOT(ajusterNombreDiabolosEnCouleur(int)));

    // Affichage des utilisés diabolos en couleur
    connect(this,SIGNAL(getCouleurDiabolos(QList<QColor>)), animation, SLOT(setCouleurDiabolos(QList<QColor>)));

    // Modification de la couleur des diabolos
    QSignalMapper *signalMapper = new QSignalMapper(this);
    connect(signalMapper, SIGNAL(mapped(int)), this, SIGNAL(getDiabolosModifie(int)));
    connect(this,SIGNAL(getDiabolosModifie(int)),this,SLOT(ajusterCouleurDesDiabolos(int)));

    for(i=0;i<9;i++)
    {
        QString text = QString::number(i);
        signalMapper->setMapping(affichageDiabolos.at(i), i);
        connect(affichageDiabolos.at(i), SIGNAL(clicked()), signalMapper, SLOT(map()));
    }


    /** Agencement de la fenêtre principale **/

    QVBoxLayout *vboxLayout = new QVBoxLayout;
    vboxLayout->addWidget(saisieSiteswap);
    vboxLayout->addLayout(hboxBoutonsRadio);
    vboxLayout->addWidget(temps);
    vboxLayout->addWidget(diabolos);
    vboxLayout->addLayout(boutons);
    vboxLayout->setAlignment(Qt::AlignLeft);
    setLayout(vboxLayout);

}


void mywidget::ajusterNombreDiabolosEnCouleur(int nbDiab)
{
    int i;

    QString style;
    for(i=0;i<nbDiab;i++)
    {
        style = "background-color: rgb(" + QString::number(couleurDiabolos.at(i).red()) + "," + QString::number(couleurDiabolos.at(i).green()) + "," + QString::number(couleurDiabolos.at(i).blue()) + ")";
        affichageDiabolos.at(i)->setStyleSheet(style);
    }

    for(i=nbDiab;i<9;i++)
    {
        style = "background-color: rgb(100,100,100)";
        affichageDiabolos.at(i)->setStyleSheet(style);
    }
}

void mywidget::ajusterCouleurDesDiabolos(int i)
{
    QColor couleur = QColorDialog::getColor(Qt::white, this);

    couleurDiabolos.replace(i,couleur);

    QString style;
    style = "background-color: rgb(" + QString::number(couleurDiabolos.at(i).red()) + "," + QString::number(couleurDiabolos.at(i).green()) + "," + QString::number(couleurDiabolos.at(i).blue()) + ")";
    affichageDiabolos.at(i)->setStyleSheet(style);

    emit getCouleurDiabolos(couleurDiabolos);
}

